﻿using $ext_safeprojectname$.InstanceHolder;
using $ext_safeprojectname$.Model;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.ChangeTracking;
using Microsoft.Extensions.Logging;
using System;
using System.Diagnostics;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;

namespace $safeprojectname$
{
    public class ApplicationDbContext : DbContext
    {
        private string _connString;
        public static readonly LoggerFactory MyLoggerFactory
            = new LoggerFactory(new[] { new DbLoggerProvider(s => Console.Write(s)) });

        #region Constructor

        public ApplicationDbContext()
        {
            _connString = Instance.AppInstance.CurrentConnectionstring;
        }

        public ApplicationDbContext(string connectionString)
        {
            _connString = connectionString;
        }

        public ApplicationDbContext(DbContextOptions options) : base(options)
        {  }


        #endregion

        #region DbSets

        //TODO: DbSets anlegen
        
        #endregion

        #region Overrides

        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            _connString = Instance.AppInstance.CurrentConnectionstring;

            if (!optionsBuilder.IsConfigured)
            {
                Debug.WriteLine($"Configured connectionstring: {_connString}");
                if (Instance.AppInstance.EfLogOn)
                {
                    optionsBuilder.UseLoggerFactory(MyLoggerFactory);
                }
                optionsBuilder.UseSqlServer(_connString);
            }
        }

        public override int SaveChanges()
        {
            UpdateEntitysBeforeSaving();
            return base.SaveChanges();
        }

        public override Task<int> SaveChangesAsync(bool acceptAllChangesOnSuccess, CancellationToken cancellationToken = default)
        {
            UpdateEntitysBeforeSaving();
            return base.SaveChangesAsync(acceptAllChangesOnSuccess, cancellationToken);
        }

        #endregion

        #region Helper Methods

        private void UpdateEntitysBeforeSaving()
        {

            var objectStateEntries = ChangeTracker.Entries()
                .Where(e => e.Entity is ModelBase && e.State != EntityState.Detached && e.State != EntityState.Unchanged).ToList();

            foreach (var entry in objectStateEntries)
            {
                if (!(entry.Entity is ModelBase entityBase)) continue;
                switch (entry.State)
                {
                    case EntityState.Deleted:
                        if (entityBase is ILogicalDelete delete)
                        {
                            entry.State = EntityState.Modified;

                            delete.DeletedTimestamp = DateTime.Now;
                            delete.DeletedFlag = true;
                            entry.Property(nameof(delete.DeletedTimestamp)).IsModified = true;
                            entry.Property(nameof(delete.DeletedFlag)).IsModified = true;
                        }
                        break;
                    case EntityState.Modified:

                        if (entityBase is ILogicalTimestamp timestamp)
                        {
                            timestamp.LastUpdateTimestamp = DateTime.Now;
                            entry.Property(nameof(timestamp.LastUpdateTimestamp)).IsModified = true;

                        }
                        if (entityBase is ILogicalDelete deleteedited)
                        {
                            string originalvalue = entry.Property(nameof(deleteedited.DeletedFlag)).OriginalValue.ToString();
                            string currentValue = entry.Property(nameof(deleteedited.DeletedFlag)).CurrentValue.ToString();
                            bool.TryParse(originalvalue, out var result);
                            bool.TryParse(currentValue, out var result1);
                            if (result != result1)
                            {
                                deleteedited.DeletedTimestamp = DateTime.Now;
                                entry.Property(nameof(deleteedited.DeletedTimestamp)).IsModified = true;
                            }
                        }
                        break;
                    case EntityState.Added:
                        if (entityBase is ILogicalTimestamp logicalTimestamp)
                        {
                            logicalTimestamp.CreationTimestamp = DateTime.Now;
                            logicalTimestamp.LastUpdateTimestamp = DateTime.Now;
                            entry.Property(nameof(logicalTimestamp.CreationTimestamp)).IsModified = true;
                            entry.Property(nameof(logicalTimestamp.LastUpdateTimestamp)).IsModified = true;
                        }
                        break;
                }

            }

        }

        public void DetachAll()
        {
            foreach (EntityEntry entityEntry in ChangeTracker.Entries().ToArray())
            {
                if (entityEntry.Entity != null)
                {
                    entityEntry.State = EntityState.Detached;
                }
            }
        }

        #endregion
    }
}
