﻿using $ext_safeprojectname$.Context;
using $ext_safeprojectname$.Model;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;

namespace $safeprojectname$
{
    public class GenericRepository<T> : IGenericRepository<T>, IDisposable where T : class
    {
        private readonly bool _disposeContext = true;
        public object Context { get; set; } = new object();

        internal ApplicationDbContext ContextInternal => (ApplicationDbContext)Context;

        #region Constructor

        public GenericRepository() : this(null, false)
        { }

        public GenericRepository(bool tracking) : this(null, tracking)
        { }

        public GenericRepository(object context, bool tracking) : this((ApplicationDbContext)context, tracking)
        { }

        internal GenericRepository(ApplicationDbContext context = null, bool tracking = false)
        {
            if (!(context == null)) Debug.WriteLine($"Create new instance of GenericProvider. ProviderName:{context.Database.ProviderName}");
            else Debug.WriteLine($"Create new instance of GenericProvider without context.");

            if (context != null)
            {
                Context = context;
                _disposeContext = false;
            }
            else
            {
                Context = new ApplicationDbContext();
            }

            ContextInternal.ChangeTracker.AutoDetectChangesEnabled = tracking;

            if (tracking) ContextInternal.ChangeTracker.QueryTrackingBehavior = QueryTrackingBehavior.TrackAll;
            else ContextInternal.ChangeTracker.QueryTrackingBehavior = QueryTrackingBehavior.NoTracking;
        }

        #endregion

        #region IGenericRepository Implementierung

        public virtual void Add(T entity)
        {
            Debug.WriteLine($"Add (entity) of Type {typeof(T).ToString()}");
            ((ApplicationDbContext)Context).Set<T>().Add(entity);
        }

        public virtual bool Any(bool includeDeleted = false)
        {
            if (!includeDeleted)
                return ((ApplicationDbContext)Context).Set<T>().Where(e => ((ILogicalDelete)e).DeletedFlag == false).Any();
            else
                return ((ApplicationDbContext)Context).Set<T>().Any();
        }

        public virtual async Task<bool> AnyAsync(bool includeDeleted = false)
        {
            if (!includeDeleted)
                return await((ApplicationDbContext)Context).Set<T>().Where(e => ((ILogicalDelete)e).DeletedFlag == false).AnyAsync().ConfigureAwait(true);
            else
                return await((ApplicationDbContext)Context).Set<T>().AnyAsync().ConfigureAwait(true);
        }

        public virtual bool Any(IQueryable<T> query)
        {
            return query.Any();
        }

        public virtual async Task<bool> AnyAsync(IQueryable<T> query)
        {
            return await query.AnyAsync();
        }

        public virtual void Attach(T entity)
        {
            Debug.WriteLine($"Try to attach (entity) of Type {typeof(T).ToString()}");
            if (((ApplicationDbContext)Context).Entry(entity).State == EntityState.Detached)
            {
                try
                {
                    ((ApplicationDbContext)Context).Attach(entity);
                }
                catch (InvalidOperationException ex)
                {
                    Debug.WriteLine(ex.Message);
                }
                Debug.WriteLine($"Successfully attached Entity of Type: {entity.GetType().ToString()}");
            }
        }

        public virtual bool CollectionLoaded(T entity, string prop)
        {
            return ((ApplicationDbContext)Context).Entry(entity).Collection(prop).IsLoaded;
        }

        public virtual int CountAll(bool includeDeleted = false)
        {
            if (!includeDeleted) return ((ApplicationDbContext)Context).Set<T>().Where(e => ((ILogicalDelete)e).DeletedFlag == false).Count();
            else return ((ApplicationDbContext)Context).Set<T>().Count();
        }

        public virtual async Task<int> CountAsync(bool includeDeleted = false)
        {
            if (!includeDeleted) return await((ApplicationDbContext)Context).Set<T>().Where(e => ((ILogicalDelete)e).DeletedFlag == false).CountAsync().ConfigureAwait(true);
            else return await((ApplicationDbContext)Context).Set<T>().CountAsync().ConfigureAwait(true);
        }

        public virtual void Delete(T entity)
        {
            Debug.WriteLine($"Delete (entity) of Type {typeof(T).ToString()}");
            ((ApplicationDbContext)Context).Set<T>().Remove(entity);
        }

        public virtual void Detach(T entity)
        {
            Debug.WriteLine($"Detach (entity) of Type {typeof(T).ToString()}");
            ((ApplicationDbContext)Context).Entry(entity).State = EntityState.Detached;
            Debug.WriteLine($"Successfully detached Entity of Type: {entity.GetType().ToString()}");
        }

        public virtual void Edit(T entity)
        {
            Debug.WriteLine($"Edit (entity) of Type {typeof(T).ToString()}");
            ((ApplicationDbContext)Context).Entry(entity).State = EntityState.Modified;
            Debug.WriteLine($"Successfully set EntryState to 'Modified'. Entity of Type: {entity.GetType().ToString()}");
        }

        public virtual T Find(int id)
        {
            Debug.WriteLine($"Find (Direct return) for T of Type {typeof(T).ToString()}");
            return ((ApplicationDbContext)Context).Set<T>().Find(id);
        }

        public virtual async Task<T> FindAsync(int id)
        {
            Debug.WriteLine($"FindAsync (Direct return) for T of Type {typeof(T).ToString()}");
            return await((ApplicationDbContext)Context).Set<T>().FindAsync(id).ConfigureAwait(true);
        }

        public virtual IQueryable<T> FindBy(Expression<Func<T, bool>> predicate, bool tracking = false)
        {
            Debug.WriteLine($"FindBy for T of Type {typeof(T).ToString()} - Tracking: {tracking.ToString()}");
            IQueryable<T> query = ((ApplicationDbContext)Context).Set<T>().Where(predicate);
            if (tracking) query = query.AsTracking();
            Debug.WriteLine("FindBy - Return Query");
            return query;
        }

        public virtual async Task<T> FindByAsync(Expression<Func<T, bool>> predicate, bool tracking = false)
        {
            Debug.WriteLine($"FindByAsync for T of Type {typeof(T).ToString()} - Tracking: {tracking.ToString()}");
            IQueryable<T> query = ((ApplicationDbContext)Context).Set<T>().Where(predicate);
            if (tracking) query = query.AsTracking();
            Debug.WriteLine("FindBy - Return Query");
            return await query.SingleOrDefaultAsync();
        }

        public virtual async Task<T> FindByAsync(IQueryable<T> query)
        {
            Debug.WriteLine($"FindByAsync for T of Type {typeof(T).ToString()}");
            return await query.SingleOrDefaultAsync();
        }

        public virtual IQueryable<T> GetAll(bool tracking = false)
        {
            Debug.WriteLine($"Calling GetAll. Tracking: {tracking.ToString()}");
            IQueryable<T> query = ((ApplicationDbContext)Context).Set<T>();
            if (tracking) query = query.AsTracking();
            Debug.WriteLine("Get All - Return Query");
            return query;
        }

        public virtual async Task<List<T>> GetAsync(IQueryable<T> query)
        {
            Debug.WriteLine($"GetAllAsync (Direct return) for T of Type {typeof(T).ToString()}");
            return await query.ToListAsync();
        }

        public virtual void LoadCollection(T entity, string prop)
        {
            Debug.WriteLine($"Try to load Collection '{entity.GetType().ToString()}' for Entity of Type {prop.ToString()}");
            if (((ApplicationDbContext)Context).Entry(entity).Collection(prop).IsLoaded)
                Debug.WriteLine($"Collection '{prop.ToString()}' is allready loaded. Abort loading reference");
            else
            {
                ((ApplicationDbContext)Context).Entry(entity).Collection(prop).Load();
                Debug.WriteLine($"Collection '{prop.ToString()}' was loaded.");
            }
        }

        public virtual void LoadReference(T entity, string prop)
        {
            Debug.WriteLine($"Try to load Reference '{entity.GetType().ToString()}' for Entity of Type {prop.ToString()}");
            if (((ApplicationDbContext)Context).Entry(entity).Reference(prop).IsLoaded)
            {
                Debug.WriteLine($"Reference '{prop.ToString()}' is allready loaded. Abort loading reference");
            }
            else
            {
                ((ApplicationDbContext)Context).Entry(entity).Reference(prop).Load();
                Debug.WriteLine($"Reference '{prop.ToString()}' was loaded.");
            }
        }

        public virtual bool ReferenceLoaded(T entity, string prop)
        {
            return ((ApplicationDbContext)Context).Entry(entity).Reference(prop).IsLoaded;
        }

        public virtual int Save()
        {
            Debug.WriteLine(GetStatistic());
            ContextInternal.ChangeTracker.DetectChanges();
            int anz = ContextInternal.SaveChanges();
            Debug.WriteLine($"GenericProvider:Save returned {anz.ToString()} changes saved successfully");
            return anz;
        }

        public virtual async Task<int> SaveAsync()
        {
            Debug.WriteLine(GetStatistic());
            ContextInternal.ChangeTracker.DetectChanges();
            int anz = await ContextInternal.SaveChangesAsync();
            Debug.WriteLine($"GenericProvider:Save returned {anz.ToString()} changes saved successfully");
            return anz;
        }

        public virtual IQueryable<T> SetTracking(bool tracking, IQueryable<T> query)
        {
            Debug.WriteLine($"SetTracking for Query of Type '{typeof(T).ToString()}' to '{tracking.ToString()}' and Return modified Query");
            if (tracking) return query.AsTracking();
            else return query.AsNoTracking();
        }

        #endregion

        #region Helper Methods

        public virtual bool HasContextChanges()
        {
            Debug.WriteLine("GenericRepository:HasContextChanges...");
            Debug.WriteLine("Detecting changes");
            if (ContextInternal == null) return false;
            ContextInternal.ChangeTracker.DetectChanges();
            if (ContextInternal.ChangeTracker.HasChanges())
            {
                Debug.WriteLine("Context:ChangeTracker has changes, returning true");
                return true;
            }
            else
            {
                Debug.WriteLine("Context:ChangeTracker has NO changes, returning false");
                return false;
            }
        }

        protected virtual string GetStatistic<TEntity>() where TEntity : class
        {
            string statistic = "";
            statistic += "Changed: " + ContextInternal.ChangeTracker.Entries<TEntity>().Where(x => x.State == EntityState.Modified).Count().ToString();
            statistic += "New: " + ContextInternal.ChangeTracker.Entries<TEntity>().Where(x => x.State == EntityState.Added).Count().ToString();
            statistic += "Deleted: " + ContextInternal.ChangeTracker.Entries<TEntity>().Where(x => x.State == EntityState.Deleted).Count().ToString();
            Debug.WriteLine(statistic);
            return statistic;
        }

        protected virtual string GetStatistic()
        {
            string statistic = "";
            statistic += "Changed: " + ContextInternal.ChangeTracker.Entries().Where(x => x.State == EntityState.Modified).Count().ToString();
            statistic += "New: " + ContextInternal.ChangeTracker.Entries().Where(x => x.State == EntityState.Added).Count().ToString();
            statistic += "Deleted: " + ContextInternal.ChangeTracker.Entries().Where(x => x.State == EntityState.Deleted).Count().ToString();
            Debug.WriteLine(statistic);
            return statistic;
        }

        public virtual TEntity GetEntityOriginalValue<TEntity>(TEntity entity) where TEntity : class
        {
            return (TEntity)ContextInternal.Entry<TEntity>(entity).OriginalValues.ToObject();
        }

        public virtual TEntity GetEntityDbValue<TEntity>(TEntity entity) where TEntity : class
        {
            return (TEntity)ContextInternal.Entry<TEntity>(entity).GetDatabaseValues().ToObject();
        }

        public virtual async Task<bool> CreateDbIfNotExist(bool doSeeding)
        {
            bool ret = await ContextInternal.Database.EnsureCreatedAsync();
            //TODO:Seed Methode erstellen(ContextInternal).ConfigureAwait(true);
            //if (doSeeding && ret) await 
            return ret;
        }

        public virtual void DetachAll()
        {
            Debug.WriteLine("Detach all Entities!!");
            ContextInternal.DetachAll();
        }
        #endregion

        #region IDisposable Implementierung

        public void Dispose()
        {
            if (_disposeContext)
            {
                Debug.WriteLine("Disposing GenericProvider...");
                ContextInternal.Dispose();
                Debug.WriteLine("GenericProvider Disposed");
            }
            else
            {
                Debug.WriteLine("Do not Dispose Context of GenericProvider because its created from another place!!");
            }
        }

        #endregion
    }
}
