﻿using $ext_safeprojectname$.Helper;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media.Animation;

namespace $safeprojectname$
{
    /// <summary>
    /// Helfer, um Framework-Elemente auf bestimmte Weise zu animieren
    /// </ summary>
    public static class FrameworkElementAnimation
    {

        #region Slide In / Out

        /// <summary>
        /// Schiebt ein Elemeint herein
        /// </ summary>
        /// <param name = "element"> Das zu animierende Element </ param>
        /// <param name = "direction"> Die Richtung zum einschieben </ param>
        /// <param name = "seconds"> Die Zeit, die die Animation benötigt </ param>
        /// <param name = "keepMargin"> Gibt an, ob das Element während der Animation die gleiche Breite haben soll. </ param>
        /// <param name = "size"> Die Breite / Höhe der Animation, auf die animiert werden soll. Wenn nicht angegeben, wird die Elementgröße verwendet </ param>
        /// <param name = "firstLoad"> Gibt an, ob dies das erste Laden ist </ param>
        /// <returns> </ returns>
        public static async Task SlideAndFadeInAsync(this FrameworkElement element, AnimationSlideInDirectionEnum direction, bool firstLoad, float seconds = 0.3f, bool keepMargin = true, int size = 0)
        {
            // Storyboard erstellen
            var sb = new Storyboard();

            // schiebe in die richtige Richtung
            switch (direction)
            {
                // einschieben von links hinzufügen
                case AnimationSlideInDirectionEnum.Left:
                    sb.AddSlideFromLeft(seconds, size == 0 ? element.ActualWidth : size, keepMargin: keepMargin);
                    break;
                // einschieben von rechts hinzufügen
                case AnimationSlideInDirectionEnum.Right:
                    sb.AddSlideFromRight(seconds, size == 0 ? element.ActualWidth : size, keepMargin: keepMargin);
                    break;
                // einschieben von oben hinzufügen
                case AnimationSlideInDirectionEnum.Top:
                    sb.AddSlideFromTop(seconds, size == 0 ? element.ActualHeight : size, keepMargin: keepMargin);
                    break;
                // einschieben von unten hinzufügen
                case AnimationSlideInDirectionEnum.Bottom:
                    sb.AddSlideFromBottom(seconds, size == 0 ? element.ActualHeight : size, keepMargin: keepMargin);
                    break;
            }
            // einblenden animation hinzufügen
            sb.AddFadeIn(seconds);

            // Animation starten
            sb.Begin(element);

            // Seite nur sichtbar machen, wenn wir sie animieren oder wenn sie zum ersten Mal geladen wird
            if (seconds != 0 || firstLoad)
                element.Visibility = Visibility.Visible;

            // auf Fertigstellung warten
            await Task.Delay((int)(seconds * 1000));
        }

        /// <summary>
        /// Schiebt ein Element heraus
        /// </ summary>
        /// <param name = "element"> Das zu animierende Element </ param>
        /// <param name = "direction"> Die Richtung des Element (dies ist für die umgekehrte Aktion zum Herausrutschen, sodass Links nach links herausrutschen würde) </ param>
        /// <param name = "seconds"> Die Zeit, die die Animation benötigt </ param>
        /// <param name = "keepMargin"> Gibt an, ob das Element während der Animation die gleiche Breite haben soll. </ param>
        /// <param name = "size"> Die Breite / Höhe der Animation, auf die animiert werden soll. Wenn nicht angegeben, wird die Elementgröße verwendet </ param>
        /// <returns> </ returns>
        public static async Task SlideAndFadeOutAsync(this FrameworkElement element, AnimationSlideInDirectionEnum direction, float seconds = 0.3f, bool keepMargin = true, int size = 0)
        {
            // Kommentare siehe oben
            var sb = new Storyboard();
            switch (direction)
            {
                case AnimationSlideInDirectionEnum.Left:
                    sb.AddSlideToLeft(seconds, size == 0 ? element.ActualWidth : size, keepMargin: keepMargin);
                    break;
                case AnimationSlideInDirectionEnum.Right:
                    sb.AddSlideToRight(seconds, size == 0 ? element.ActualWidth : size, keepMargin: keepMargin);
                    break;
                case AnimationSlideInDirectionEnum.Top:
                    sb.AddSlideToTop(seconds, size == 0 ? element.ActualHeight : size, keepMargin: keepMargin);
                    break;
                case AnimationSlideInDirectionEnum.Bottom:
                    sb.AddSlideToBottom(seconds, size == 0 ? element.ActualHeight : size, keepMargin: keepMargin);
                    break;
            }
            sb.AddFadeOut(seconds);
            sb.Begin(element);
            if (seconds != 0)
                element.Visibility = Visibility.Visible;
            await Task.Delay((int)(seconds * 1000));
            if (element.Opacity == 0)
                element.Visibility = Visibility.Hidden;
        }

        #endregion

        #region Fade In / Out

        /// <summary>
        /// Blendet ein Element ein
        /// </ summary>
        /// <param name = "element"> Das zu animierende Element </ param>
        /// <param name = "seconds"> Die Zeit, die die Animation benötigt </ param>
        /// <param name = "firstLoad"> Gibt an, ob dies das erste Laden ist </ param>
        /// <returns> </ returns>
        public static async Task FadeInAsync(this FrameworkElement element, bool firstLoad, float seconds = 0.3f)
        {
            var sb = new Storyboard();
            sb.AddFadeIn(seconds);
            sb.Begin(element);
            if (seconds != 0 || firstLoad)
                element.Visibility = Visibility.Visible;
            await Task.Delay((int)(seconds * 1000));
        }

        /// <summary>
        /// Blendet ein Element aus
        /// </ summary>
        /// <param name = "element"> Das zu animierende Element </ param>
        /// <param name = "seconds"> Die Zeit, die die Animation benötigt </ param>
        /// <param name = "firstLoad"> Gibt an, ob dies das erste Laden ist </ param>
        /// <returns> </ returns>
        public static async Task FadeOutAsync(this FrameworkElement element, float seconds = 0.3f)
        {
            var sb = new Storyboard();
            sb.AddFadeOut(seconds);
            sb.Begin(element);
            if (seconds != 0)
                element.Visibility = Visibility.Visible;
            await Task.Delay((int)(seconds * 1000));
            element.Visibility = Visibility.Collapsed;
        }
        #endregion
    }
}
