﻿using $ext_safeprojectname$.Helper;
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;

namespace $safeprojectname$
{
    #region Focus Properties

    /// <summary>
    /// Fokussiert (Tastaturfokus) dieses Element beim Laden
    /// </ summary>
    public class IsFocusedProperty : AttachedPropertyBase<IsFocusedProperty, bool>
    {
        public override void OnValueChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            if (!(sender is Control control)) return;
            control.Loaded += (s, se) => control.Focus();
        }
    }

    /// <summary>
    /// Fokussiert (Tastaturfokus) dieses Element, wenn true
    /// </ summary>
    public class FocusProperty : AttachedPropertyBase<FocusProperty, bool>
    {
        public override void OnValueChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            if (!(sender is Control control)) return;

            if ((bool)e.NewValue) control.Focus();
        }
    }

    /// <summary>
    /// Fokussiert (Tastaturfokus) und wählt den gesamten Text in diesem Element aus, wenn true
    /// </ summary>
    public class FocusAndSelectProperty : AttachedPropertyBase<FocusAndSelectProperty, bool>
    {
        public override void OnValueChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            if ((sender is TextBoxBase control))
            {
                if ((bool)e.NewValue)
                {
                    control.Focus();
                    control.SelectAll();
                }
            }
            if ((sender is PasswordBox pass))
            {
                if ((bool)e.NewValue)
                {
                    pass.Focus();
                    pass.SelectAll();
                }
            }
        }
    }
    #endregion

    #region Numeric Properties

    public class OnlyNumericProperty : AttachedPropertyBase<OnlyNumericProperty,NumericFormatEnum>
    {
        public override void OnValueChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            if(!(sender is TextBox textBox))
                throw new ApplicationException("Attached property must be used with TextBox.");

            var castedValue = (NumericFormatEnum?)e.NewValue;
            if(castedValue.HasValue)
            {
                textBox.PreviewTextInput += TextBoxMethods.PreviewTextInput;
                DataObject.AddPastingHandler(textBox, TextBoxMethods.PasteEventHandler);
            }
            else
            {
                textBox.PreviewTextInput -= TextBoxMethods.PreviewTextInput;
                DataObject.RemovePastingHandler(textBox, TextBoxMethods.PasteEventHandler);
            }

        }
    }

    public class DefaultValueProperty : AttachedPropertyBase<DefaultValueProperty, string>
    {
        public override void OnValueChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            if (!(sender is TextBox textBox))
                throw new ApplicationException("Attached property must be used with TextBox.");

            var castedValue = (string)e.NewValue;
            if (castedValue != null)
                textBox.TextChanged += TextBoxMethods.TextChanged;
            else
                textBox.TextChanged -= TextBoxMethods.TextChanged;
        }
    }

    public class EvenOddConstraintProperty : AttachedPropertyBase<EvenOddConstraintEnum, EvenOddConstraintEnum>
    { }

    internal static class TextBoxMethods
    {
        public static void PreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            var textBox = (TextBox)sender;

            string newText;

            if (textBox.SelectionLength == 0)
            {
                newText = textBox.Text.Insert(textBox.SelectionStart, e.Text);
            }
            else
            {
                var textAfterDelete = textBox.Text.Remove(textBox.SelectionStart, textBox.SelectionLength);

                newText = textAfterDelete.Insert(textBox.SelectionStart, e.Text);
            }

            var evenOddConstraint = EvenOddConstraintProperty.GetValue(textBox);

            switch (OnlyNumericProperty.GetValue(textBox))
            {
                case NumericFormatEnum.Double:
                    {
                        if (double.TryParse(newText, out double number))
                        {
                            switch (evenOddConstraint)
                            {
                                case EvenOddConstraintEnum.OnlyEven:

                                    if (number % 2 != 0)
                                        e.Handled = true;
                                    else
                                        e.Handled = false;

                                    break;

                                case EvenOddConstraintEnum.OnlyOdd:

                                    if (number % 2 == 0)
                                        e.Handled = true;
                                    else
                                        e.Handled = false;

                                    break;
                            }
                        }
                        else
                            e.Handled = true;

                        break;
                    }

                case NumericFormatEnum.Int:
                    {
                        if (int.TryParse(newText, out int number))
                        {
                            switch (evenOddConstraint)
                            {
                                case EvenOddConstraintEnum.OnlyEven:

                                    if (number % 2 != 0)
                                        e.Handled = true;
                                    else
                                        e.Handled = false;

                                    break;

                                case EvenOddConstraintEnum.OnlyOdd:

                                    if (number % 2 == 0)
                                        e.Handled = true;
                                    else
                                        e.Handled = false;

                                    break;
                            }
                        }
                        else
                            e.Handled = true;

                        break;
                    }

                case NumericFormatEnum.Uint:
                    {
                        if (uint.TryParse(newText, out uint number))
                        {
                            switch (evenOddConstraint)
                            {
                                case EvenOddConstraintEnum.OnlyEven:

                                    if (number % 2 != 0)
                                        e.Handled = true;
                                    else
                                        e.Handled = false;

                                    break;

                                case EvenOddConstraintEnum.OnlyOdd:

                                    if (number % 2 == 0)
                                        e.Handled = true;
                                    else
                                        e.Handled = false;

                                    break;
                            }
                        }
                        else
                            e.Handled = true;

                        break;
                    }

                case NumericFormatEnum.Natural:
                    {
                        if (uint.TryParse(newText, out uint number))
                        {
                            if (number == 0)
                                e.Handled = true;
                            else
                            {
                                switch (evenOddConstraint)
                                {
                                    case EvenOddConstraintEnum.OnlyEven:

                                        if (number % 2 != 0)
                                            e.Handled = true;
                                        else
                                            e.Handled = false;

                                        break;

                                    case EvenOddConstraintEnum.OnlyOdd:

                                        if (number % 2 == 0)
                                            e.Handled = true;
                                        else
                                            e.Handled = false;

                                        break;
                                }
                            }
                        }
                        else
                            e.Handled = true;

                        break;
                    }
            }
        }     


        public static void PasteEventHandler(object sender, DataObjectPastingEventArgs e)
        {
            var textBox = (TextBox)sender;

            if (e.DataObject.GetDataPresent(typeof(string)))
            {
                var clipboardText = (string)e.DataObject.GetData(typeof(string));

                var newText = textBox.Text.Insert(textBox.SelectionStart, clipboardText);

                var evenOddConstraint = EvenOddConstraintProperty.GetValue(textBox);

                switch (OnlyNumericProperty.GetValue(textBox))
                {
                    case NumericFormatEnum.Double:
                        {
                            if (double.TryParse(newText, out double number))
                            {
                                switch (evenOddConstraint)
                                {
                                    case EvenOddConstraintEnum.OnlyEven:

                                        if (number % 2 != 0)
                                            e.CancelCommand();

                                        break;

                                    case EvenOddConstraintEnum.OnlyOdd:

                                        if (number % 2 == 0)
                                            e.CancelCommand();

                                        break;
                                }
                            }
                            else
                                e.CancelCommand();

                            break;
                        }

                    case NumericFormatEnum.Int:
                        {
                            if (int.TryParse(newText, out int number))
                            {
                                switch (evenOddConstraint)
                                {
                                    case EvenOddConstraintEnum.OnlyEven:

                                        if (number % 2 != 0)
                                            e.CancelCommand();

                                        break;

                                    case EvenOddConstraintEnum.OnlyOdd:

                                        if (number % 2 == 0)
                                            e.CancelCommand();


                                        break;
                                }
                            }
                            else
                                e.CancelCommand();

                            break;
                        }

                    case NumericFormatEnum.Uint:
                        {
                            if (uint.TryParse(newText, out uint number))
                            {
                                switch (evenOddConstraint)
                                {
                                    case EvenOddConstraintEnum.OnlyEven:

                                        if (number % 2 != 0)
                                            e.CancelCommand();

                                        break;

                                    case EvenOddConstraintEnum.OnlyOdd:

                                        if (number % 2 == 0)
                                            e.CancelCommand();


                                        break;
                                }
                            }
                            else
                                e.CancelCommand();

                            break;
                        }

                    case NumericFormatEnum.Natural:
                        {
                            if (uint.TryParse(newText, out uint number))
                            {
                                if (number == 0)
                                    e.CancelCommand();
                                else
                                {
                                    switch (evenOddConstraint)
                                    {
                                        case EvenOddConstraintEnum.OnlyEven:

                                            if (number % 2 != 0)
                                                e.CancelCommand();

                                            break;

                                        case EvenOddConstraintEnum.OnlyOdd:

                                            if (number % 2 == 0)
                                                e.CancelCommand();

                                            break;
                                    }
                                }
                            }
                            else
                            {
                                e.CancelCommand();
                            }

                            break;
                        }
                }
            }
            else
            {
                e.CancelCommand();
            }
        }

        public static void TextChanged(object sender, TextChangedEventArgs e)
        {
            var textBox = (TextBox)sender;

            var defaultValue = DefaultValueProperty.GetValue(textBox);

            var evenOddConstraint = EvenOddConstraintProperty.GetValue(textBox);

            switch (OnlyNumericProperty.GetValue(textBox))
            {
                case NumericFormatEnum.Double:
                    {
                        if (double.TryParse(textBox.Text, out double number))
                        {
                            switch (evenOddConstraint)
                            {
                                case EvenOddConstraintEnum.OnlyEven:

                                    if (number % 2 != 0)
                                        textBox.Text = defaultValue;

                                    break;

                                case EvenOddConstraintEnum.OnlyOdd:

                                    if (number % 2 == 0)
                                        textBox.Text = defaultValue;

                                    break;
                            }
                        }
                        else
                            textBox.Text = defaultValue;

                        break;
                    }

                case NumericFormatEnum.Int:
                    {
                        if (int.TryParse(textBox.Text, out int number))
                        {
                            switch (evenOddConstraint)
                            {
                                case EvenOddConstraintEnum.OnlyEven:

                                    if (number % 2 != 0)
                                        textBox.Text = defaultValue;

                                    break;

                                case EvenOddConstraintEnum.OnlyOdd:

                                    if (number % 2 == 0)
                                        textBox.Text = defaultValue;

                                    break;
                            }
                        }
                        else
                            textBox.Text = defaultValue;

                        break;
                    }

                case NumericFormatEnum.Uint:
                    {
                        if (uint.TryParse(textBox.Text, out uint number))
                        {
                            switch (evenOddConstraint)
                            {
                                case EvenOddConstraintEnum.OnlyEven:

                                    if (number % 2 != 0)
                                        textBox.Text = defaultValue;

                                    break;

                                case EvenOddConstraintEnum.OnlyOdd:

                                    if (number % 2 == 0)
                                        textBox.Text = defaultValue;

                                    break;
                            }
                        }
                        else
                            textBox.Text = defaultValue;

                        break;
                    }

                case NumericFormatEnum.Natural:
                    {
                        if (uint.TryParse(textBox.Text, out uint number))
                        {
                            if (number == 0)
                                textBox.Text = defaultValue;
                            else
                            {
                                switch (evenOddConstraint)
                                {
                                    case EvenOddConstraintEnum.OnlyEven:

                                        if (number % 2 != 0)
                                            textBox.Text = defaultValue;

                                        break;

                                    case EvenOddConstraintEnum.OnlyOdd:

                                        if (number % 2 == 0)
                                            textBox.Text = defaultValue;

                                        break;
                                }
                            }
                        }
                        else
                        {
                            textBox.Text = defaultValue;
                        }

                        break;
                    }
            }
        }
        
    }

    #endregion
}
